@@ -279,25 +279,12 @@ btctl_controller_impl_cmd_scan(BtctlController *bc)
 	bdaddr_t bdaddr;
 	char name[248];
 	int i, dd;
-	int dev_id=0;
 
 	length  = 8;  /* ~10 seconds */
 	num_rsp = 10;
 	flags = 0;
 
-	dev_id=hci_devid("hci0");
-
-	if (dev_id < 0) {
-		dev_id = hci_get_route(&bdaddr);
-		if (dev_id < 0) {
-			perror("HCI device not available.");
-			/* TODO: report this status to the app */
-			btctl_controller_report_status(bc, BTCTL_STATUS_ERROR);
-			return;
-		}
-	}
-
-	printf("dev_id is %d\n", dev_id);
+	printf("dev_id is %d\n", bc->hci_id);
 
 	if (bc->cancel) {
 		btctl_controller_report_status(bc, BTCTL_STATUS_ERROR);
@@ -306,7 +293,7 @@ btctl_controller_impl_cmd_scan(BtctlController *bc)
 
 	btctl_controller_report_status(bc, BTCTL_STATUS_SCANNING);
 
-	num_rsp = hci_inquiry(dev_id, length, num_rsp, NULL, &info, flags);
+	num_rsp = hci_inquiry(bc->hci_id, length, num_rsp, NULL, &info, flags);
 
 	if (num_rsp < 0) {
 		perror("Inquiry failed.");
@@ -337,7 +324,7 @@ btctl_controller_impl_cmd_scan(BtctlController *bc)
 	}
 
 	btctl_controller_report_status(bc, BTCTL_STATUS_GETTING_NAMES);
-	dd = hci_open_dev(dev_id);
+	dd = hci_open_dev(bc->hci_id);
 	if (dd>=0) {
 		for (i = 0; i < num_rsp; i++) {
 			memset(name, 0, sizeof(name));
@@ -580,7 +567,8 @@ btctl_controller_impl_establish_rfcomm_connection(BtctlController *bc, const gch
 	struct rfcomm_dev_req *req;
 	struct rfcomm_dev_list_req *dl;
 	struct sockaddr_rc laddr, raddr;
-	int sk, alen;
+	int sk;
+	guint alen;
 	int ctl;
 
 	ctl=socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_RFCOMM);
@@ -808,23 +796,10 @@ btctl_controller_impl_get_signal_strength (BtctlController *bc,
 	struct hci_conn_info_req *cr;
 	read_rssi_rp rp;
 	int dd;
-	int dev_id = 0;
 
 	str2ba(str_bdaddr, &bdaddr);
-	dev_id = hci_devid("hci0");
 
-	if (dev_id < 0)
-	{
-		dev_id = hci_get_route(&bdaddr);
-		if (dev_id < 0)
-		{
-			g_set_error (err, 0, 0,
-					_("No bluetooth adapter found"));
-			return 0;
-		}
-	}
-
-	dd = hci_open_dev(dev_id);
+	dd = hci_open_dev(bc->hci_id);
 	if (dd < 0)
 	{
 		g_set_error (err, 0, 0,
@@ -835,6 +810,7 @@ btctl_controller_impl_get_signal_strength (BtctlController *bc,
 	if (hci_create_connection(dd, &bdaddr,
 				0x0008 | 0x0010, 0, 0, &handle, 25000) < 0)
 	{
+		perror("hci_create_connection:");
 		g_set_error (err, 0, 0,
 				_("Couldn't create a connection to the device"));
 		close (dd);
@@ -1006,7 +982,7 @@ name_complete(BtctlController *bc, int dd, unsigned char *buf, int len)
 
 	ba2str(&evt->bdaddr, addr);
 	if (!evt->status) {
-		btctl_controller_got_device_name(bc, addr, evt->name);
+		btctl_controller_got_device_name(bc, addr, (char *)evt->name);
 	} else {
 		/* some error happened: we'll report the name as the bdaddr */
 		btctl_controller_got_device_name(bc, addr, addr);
@@ -1060,6 +1036,100 @@ cb_hci_data (BtctlDiscoverySource *source, gpointer user_data)
 	return TRUE;
 }
 
+gboolean
+btctl_controller_impl_get_discoverable(BtctlController *bc, GError **err)
+{
+	static struct hci_dev_info di;
+	int ctl;
+
+	/* Open HCI socket  */
+	if ((ctl = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI)) < 0) {
+		g_set_error (err, btctl_error_quark(),
+			     BTCTL_ERROR_NO_BLUETOOTH_DEVICE,
+			     "Can't open HCI socket: %s (%s)",
+			     strerror(errno), errno);
+		return FALSE;
+	}
+	
+	di.dev_id = bc->hci_id;
+	if (ioctl(ctl, HCIGETDEVINFO, (void *) &di) < 0) {
+		g_set_error (err, btctl_error_quark(),
+			     BTCTL_ERROR_NO_BLUETOOTH_DEVICE,
+			     "Can't get device info on hci%d: %s (%d)\n",
+			     bc->hci_id, strerror(errno), errno);
+		close (ctl);
+		return FALSE;
+	}
+	close (ctl);
+
+	return hci_test_bit (HCI_ISCAN, &di.flags) ? TRUE : FALSE;
+}
+
+void
+btctl_controller_impl_set_discoverable(BtctlController *bc,
+				       gboolean discoverable,
+				       GError **err)
+{
+	struct hci_dev_req dr;
+	int ctl;
+	
+	/* Open HCI socket  */
+	if ((ctl = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI)) < 0) {
+		g_set_error (err, btctl_error_quark(),
+			     BTCTL_ERROR_NO_BLUETOOTH_DEVICE,
+			     "Can't open HCI socket: %s (%s)",
+			     strerror(errno), errno);
+		return;
+	}
+	
+	dr.dev_id  = bc->hci_id;
+	dr.dev_opt = SCAN_PAGE;
+	if (discoverable)
+		dr.dev_opt |= SCAN_INQUIRY;
+
+	if (ioctl(ctl, HCISETSCAN, (unsigned long) &dr) < 0) {
+		g_set_error (err, btctl_error_quark(),
+			     BTCTL_ERROR_NO_BLUETOOTH_DEVICE,
+			     "Can't set scan mode on hci%d: %s (%d)",
+			     bc->hci_id, strerror(errno), errno);
+		close (ctl);
+		return;
+	}
+	close (ctl);
+}
+
+void
+btctl_controller_impl_set_hci_device(BtctlController *bc,
+				     const char *hci_device,
+				     GError **err)
+{
+	if (hci_device) {
+		bc->hci_id = hci_devid(hci_device);
+		if (bc->hci_id < 0) {
+			g_set_error (err, btctl_error_quark(),
+				     BTCTL_ERROR_NO_BLUETOOTH_DEVICE,
+				     "Can't get id of hci device %s",
+				     hci_device);
+			return;
+		}
+		return;
+	}
+
+	bc->hci_id = hci_devid("hci0");
+	if (bc->hci_id < 0) {
+		/* FIXME: NULL means BDADDR_ANY? */
+		bc->hci_id = hci_get_route(NULL);
+		
+		if (bc->hci_id < 0) {
+			g_set_error (err, btctl_error_quark(),
+				     BTCTL_ERROR_NO_BLUETOOTH_DEVICE,
+				     "Can't get device id of hci0");
+			return;
+		}
+	}
+}
+
+
 void
 btctl_controller_impl_init_source(BtctlController *bc)
 {
